@@ -1,19 +1,29 @@
 package ZMQ::FFI;
-# ABSTRACT: zeromq bindings using libffi and FFI::Raw
-$ZMQ::FFI::VERSION = '0.19';
-use ZMQ::FFI::Util qw(zmq_soname zmq_version);
+$ZMQ::FFI::VERSION = '1.00';
+# ABSTRACT: version agnostic Perl bindings for zeromq using ffi
+
+use strict;
+use warnings;
+
+use ZMQ::FFI::Util qw(zmq_soname zmq_version valid_soname);
 use ZMQ::FFI::ErrorHelper;
 use Carp;
 
 sub new {
-    my $self = shift;
-    my %args = @_;
+    my ($self, %args) = @_;
 
-    $args{soname} //= zmq_soname( die => 1 );
+    if ($args{soname}) {
+        unless ( valid_soname($args{soname}) ) {
+            die "Failed to load '$args{soname}', is it on your loader path?";
+        }
+    }
+    else {
+        $args{soname} = zmq_soname( die => 1 );
+    }
 
-    # explicitly passing in a loaded error helper instance
-    # (i.e. zmq error bindings) guards against the OS X loader clobbering errno,
-    # which can happen if the bindings are loaded lazily
+    # explicitly passing in a loaded error helper instance (i.e. zmq error
+    # bindings) guards against the OS X loader clobbering errno, which can
+    # happen if the bindings are loaded lazily
     $args{error_helper} =
         ZMQ::FFI::ErrorHelper->new( soname => $args{soname} );
 
@@ -39,11 +49,11 @@ __END__
 
 =head1 NAME
 
-ZMQ::FFI - zeromq bindings using libffi and FFI::Raw
+ZMQ::FFI - version agnostic Perl bindings for zeromq using ffi
 
 =head1 VERSION
 
-version 0.19
+version 1.00
 
 =head1 SYNOPSIS
 
@@ -87,7 +97,6 @@ version 0.19
     # all topics
     {
         $s->subscribe('');
-        $p->send('ohhai');
 
         until ($s->has_pollin) {
             # compensate for slow subscriber
@@ -106,9 +115,6 @@ version 0.19
         $s->subscribe('topic1');
         $s->subscribe('topic2');
 
-        $p->send('topic1 ohhai');
-        $p->send('topic2 ohhai');
-
         until ($s->has_pollin) {
             usleep 100_000;
             $p->send('topic1 ohhai');
@@ -214,12 +220,21 @@ of the underlying libzmq version.  Where semantics differ, it will dispatch to
 the appropriate backend for you.  As it uses ffi, there is no dependency on XS
 or compilation.
 
+As of 1.00 ZMQ::FFI is implemented using L<FFI::Platypus>. This version has
+substantial performance improvements and you are encouraged to use 1.00 or
+newer.
+
 =head1 CONTEXT API
 
 =head2 new([threads, max_sockets, soname])
 
+    ZMQ::FFI->new()
+
     ZMQ::FFI->new( threads => 42, max_sockets => 42 )
 
+    ZMQ::FFI->new( soname => '/path/to/libzmq.so' )
+    ZMQ::FFI->new( soname => 'libzmq.so.3' )
+
 returns a new context object, appropriate for the version of
 libzmq found on your system. It accepts the following optional attributes:
 
@@ -267,19 +282,23 @@ I<requires zmq E<gt>= 3.x>
 
 set a context option value
 
-=head2 socket($type)
+=head2 $socket = socket($type)
 
     $ctx->socket(ZMQ_REQ)
 
 returns a socket of the specified type. See L<SOCKET API> below
 
-=head2 proxy($frontend, $backend, $capture)
+=head2 proxy($frontend, $backend, [$capture])
 
-sets up and runs a zmq_proxy, C<$capture> is optional
+sets up and runs a C<zmq_proxy>. For zmq 2.x this will use a C<ZMQ_STREAMER>
+device to simulate the proxy. The optional C<$capture> is only supported for
+zmq E<gt>= 3.x however
 
 =head2 device($type, $frontend, $backend)
 
-sets up and runs a zmq_device with specified frontend and backend sockets
+I<zmq 2.x only>
+
+sets up and runs a C<zmq_device> with specified frontend and backend sockets
 
 =head2 destroy()
 
@@ -294,8 +313,8 @@ For core attributes and functions, common across all versions of zeromq,
 convenience methods are provided. Otherwise, generic get/set methods are
 provided that will work independent of version.
 
-As attributes are constantly being added/removed from zeromq, it is unlikely the
-'static' accessors will grow much beyond the current set.
+As attributes are constantly being added/removed from zeromq, it is unlikely
+the 'static' accessors will grow much beyond the current set.
 
 =head2 ($major, $minor, $patch) = version()
 
@@ -345,6 +364,8 @@ remove C<$topic> from the subscription list
 
     $socket->send('ohhai')
 
+    $socket->send('ohhai', ZMQ_DONTWAIT)
+
 sends a message using the optional flags
 
 =head2 send_multipart($parts_aref, [$flags])
@@ -354,9 +375,13 @@ sends a message using the optional flags
 given an array ref of message parts, sends the multipart message using the
 optional flags. ZMQ_SNDMORE semantics are handled for you
 
-=head2 recv([$flags])
+=head2 $msg = recv([$flags])
 
-receive a message
+    $socket->recv()
+
+    $socket->recv(ZMQ_DONTWAIT)
+
+receives a message using the optional flags
 
 =head2 @parts = recv_multipart([$flags])
 
@@ -368,7 +393,7 @@ semantics are handled for you
 checks ZMQ_EVENTS for ZMQ_POLLIN and ZMQ_POLLOUT respectively, and returns
 true/false depending on the state
 
-=head2 get($option, $option_type)
+=head2 $value = get($option, $option_type)
 
     $socket->get(ZMQ_LINGER, 'int')
 
@@ -389,12 +414,39 @@ gets reaped
 
 =head1 ERROR HANDLING
 
-ZMQ::FFI checks the return codes of underlying zmq functions for you, and in the
-case of an error it will die with the plain english system error message.
+ZMQ::FFI checks the return codes of underlying zmq functions for you, and in
+the case of an error it will die with the plain english system error message.
 
     $ctx->socket(-1);
     # dies with 'zmq_socket: Invalid argument'
 
+=head1 FFI VS XS PERFORMANCE
+
+ZMQ::FFI uses L<FFI::Platypus> on the backend. In addition to a friendly,
+usable interface, FFI::Platypus's killer feature is C<attach>. C<attach> makes
+it possible to bind ffi functions in memory as first class Perl xsubs. This
+results in dramatic performance gains and gives you the flexibility of ffi
+with performance approaching that of XS.
+
+Testing indicates FFI::Platypus xsubs are around 30% slower than "real" XS
+xsubs. That may sound like a lot, but to put it in perspective that means, for
+zeromq, the XS bindings can send 10 million messages 1-2 seconds faster than
+the ffi ones.
+
+If you really care about 1-2 seconds over 10 million messages you should be
+writing your solution in C anyways. An equivalent C implementation will be
+several I<hundred> percent faster or more.
+
+Keep in mind also that the small speed bump you get using XS can easily be
+wiped out by crappy and poorly optimized Perl code.
+
+Now that Perl finally has a great ffi interface, it is hard to make the case
+to continue using XS. The slight speed bump just isn't worth giving up the
+convenience, flexibility, and portability of ffi.
+
+You can find the detailed performance results that informed this section at:
+L<https://gist.github.com/calid/17df5bcfb81c83786d6f>
+
 =head1 SEE ALSO
 
 =over 4
@@ -409,8 +461,16 @@ L<ZMQ::FFI::Util>
 
 =item *
 
+L<FFI::Platypus>
+
+=item *
+
 L<FFI::Raw>
 
+=item *
+
+L<ZMQ::LibZMQ3>
+
 =back
 
 =head1 AUTHOR
