@@ -1,7 +1,13 @@
 package ZMQ::FFI::ContextRole;
-$ZMQ::FFI::ContextRole::VERSION = '0.19';
+$ZMQ::FFI::ContextRole::VERSION = '1.00';
 use Moo::Role;
 
+# real underlying zmq ctx pointer
+has _ctx => (
+    is      => 'rw',
+    default => -1,
+);
+
 has soname => (
     is       => 'ro',
     required => 1,
@@ -40,7 +46,7 @@ ZMQ::FFI::ContextRole
 
 =head1 VERSION
 
-version 0.19
+version 1.00
 
 =head1 AUTHOR
 
