@@ -1,5 +1,5 @@
 package ZMQ::FFI::ErrorHandler;
-$ZMQ::FFI::ErrorHandler::VERSION = '0.19';
+$ZMQ::FFI::ErrorHandler::VERSION = '1.00';
 use Moo::Role;
 
 requires q(soname);
@@ -11,6 +11,8 @@ has error_helper => (
     handles => [qw(
         check_error
         check_null
+        bad_version
+        fatal
     )],
 );
 
@@ -28,7 +30,7 @@ ZMQ::FFI::ErrorHandler
 
 =head1 VERSION
 
-version 0.19
+version 1.00
 
 =head1 AUTHOR
 
