@@ -1,5 +1,7 @@
 package ZMQ::FFI::SocketRole;
-$ZMQ::FFI::SocketRole::VERSION = '0.19';
+$ZMQ::FFI::SocketRole::VERSION = '1.00';
+use FFI::Platypus;
+
 use Moo::Role;
 
 has soname => (
@@ -20,6 +22,28 @@ has type => (
     required => 1,
 );
 
+# real underlying zmq socket pointer
+has _socket => (
+    is      => 'rw',
+    default => -1,
+);
+
+has sockopt_sizes => (
+    is      => 'ro',
+    lazy    => 1,
+    builder => '_build_sockopt_sizes'
+);
+
+sub _build_sockopt_sizes {
+    my $ffi = FFI::Platypus->new();
+
+    return {
+        int    => $ffi->sizeof('int'),
+        sint64 => $ffi->sizeof('sint64'),
+        uint64 => $ffi->sizeof('uint64'),
+    };
+}
+
 requires qw(
     connect
     disconnect
@@ -57,7 +81,7 @@ ZMQ::FFI::SocketRole
 
 =head1 VERSION
 
-version 0.19
+version 1.00
 
 =head1 AUTHOR
 
