@@ -1,5 +1,5 @@
 package ZMQ::FFI::Versioner;
-$ZMQ::FFI::Versioner::VERSION = '0.19';
+$ZMQ::FFI::Versioner::VERSION = '1.00';
 use Moo::Role;
 
 use ZMQ::FFI::Util qw(zmq_version);
@@ -9,11 +9,15 @@ requires q(soname);
 has _version_parts => (
     is      => 'ro',
     lazy    => 1,
-    default => sub { [zmq_version(shift->soname)] }
+    default => sub { [zmq_version($_[0]->soname)] }
 );
 
 sub version {
-    return @{shift->_version_parts};
+    return @{$_[0]->_version_parts};
+}
+
+sub verstr {
+    return join('.', $_[0]->version);
 }
 
 1;
@@ -30,7 +34,7 @@ ZMQ::FFI::Versioner
 
 =head1 VERSION
 
-version 0.19
+version 1.00
 
 =head1 AUTHOR
 
